import json
import logging

from datacontract.data_contract import DataContract
import click
import requests

from src.settings import SCHEMA_REGISTRY_HOST, SCHEMA_REGISTRY_PORT

# from src import commands


log = logging.getLogger(name="").getChild(suffix=__name__)


# # Получить список всех subjects
# curl -X GET http://localhost:8081/subjects
# nshokurov@MB-YLV2KQ4C ~
# % curl -X GET http://localhost:8081/subjects                                                    [2025-09-03 15:56:37]
# ["persons-topic-value","your_topic-value"]%    

# # Получить все версии схемы для subject
# curl -X GET http://localhost:8081/subjects/vertica_datacontract/versions

# # Получить конкретную версию схемы
# curl -X GET http://localhost:8081/subjects/vertica_datacontract/versions/1
# echo $(curl -X GET http://localhost:8081/subjects/vertica_datacontract/versions/1) 

class SchemaRegistry:

    def __init__(
        self,
    ):
        ...

    def publush(
        self,
        filename: str,
        subject_name: str = "vertica_datacontract",
    ):  

        data_contract: DataContract = DataContract(
            data_contract_file=f"{filename}.yaml"
        )
        data_contract.lint()

        # Запрос на регистрацию схемы
        response: requests.Response = requests.post(
            url=f"{SCHEMA_REGISTRY_HOST}:{SCHEMA_REGISTRY_PORT}/subjects/{subject_name}/versions",
            headers={
            "Content-Type": "application/vnd.schemaregistry.v1+json"
        },
            json={
                "schemaType": "PROTOBUF",
                "schema": data_contract.export(export_format="protobuf")["protobuf"]
        }, timeout=200
        )
        click.echo(message=response.url)
        click.echo(message=response.status_code)

    def validate_custom(self, filename: str):
        

        data_contract: DataContract = DataContract(
            data_contract_file=f"{filename}.yaml"
        )
        data_contract.lint()

        click.echo(message=f" hello is custom validate {filename}")


    def validate_schema_registry(
        self,
        subject_name: str,
        filename: str,
        version: str = "latest",
    ):

        data_contract: DataContract = DataContract(
            data_contract_file=f"{filename}.yaml"
        )
        data_contract.lint()
        exported: dict = data_contract.export(export_format="protobuf")
        print(exported["protobuf"])


        response: requests.Response = requests.post(
            url=f"{SCHEMA_REGISTRY_HOST}:{SCHEMA_REGISTRY_PORT}/compatibility/subjects/{subject_name}/versions/{version}",
            headers={"Content-Type": "application/vnd.schemaregistry.v1+json"},
            data=json.dumps(obj={"schema": exported["protobuf"], "schemaType": "PROTOBUF"}),
            timeout=20,
        )
        click.echo(message=response.text)





        # import vertica_datacontract_pb2

        # # uv run python -m src publish-schema-registry --fields '{"col1": 1}'
        # schema_registry_conf: dict = {
        #     "url": f"{SCHEMA_REGISTRY_HOST}:{SCHEMA_REGISTRY_PORT}"
        # }
        # schema_registry_client: SchemaRegistryClient = SchemaRegistryClient(
        #     conf=schema_registry_conf
        # )
        # print(vertica_datacontract_pb2.Issue)
        # protobuf_serializer: ProtobufSerializer = ProtobufSerializer(vertica_datacontract_pb2.Issue, schema_registry_client)
        # print(protobuf_serializer._schema)
        # print(protobuf_serializer._schema_id)
        
        # protobuf_schema = protobuf_serializer
        # print(protobuf_schema)
            
        # Регистрация схемы вручную
        #subject = 'your_topic-value'  # Обычно схема регистрируется на теме с суффиксом "-value"
        # schema_id = schema_registry_client.register_schema(subject, protobuf_schema)

        # producer_conf = {
        #     'bootstrap.servers': bootstrap_servers,
        #     'key.serializer': None,  # если ключ не нужен или сериализатор свой
        #     'value.serializer': protobuf_serializer
        # }

        # schema_str: str = f"""
        # {
        # "type": "record",
        # "name": "User",
        # "namespace": namespace,
        # "fields": fields
        # }
        # """

        # schema_id: int = schema_registry_client.register_schema(
        #     subject_name=subject_name, schema=AvroSchema(schema_str)
        # )
        # # protobuf_schema_id = client.register_schema("subject-proto", ProtobufSchema(proto_schema_str))
        # log.info(f"Schema published with ID: {schema_id}")


# # Публикация схемы
# curl -X POST -H "Content-Type: application/vnd.schemaregistry.v1+json" \
#   --data '{
#     "schema": "{\"type\":\"record\",\"name\":\"User\",\"namespace\":\"com.example.avro\",\"fields\":[{\"name\":\"id\",\"type\":\"int\"},{\"name\":\"name\",\"type\":\"string\"},{\"name\":\"email\",\"type\":\"string\"},{\"name\":\"created_at\",\"type\":{\"type\":\"long\",\"logicalType\":\"timestamp-millis\"}}]}"
#   }' \
#   http://localhost:8081/subjects/users-value/versions

# # Ответ: {"id":1}


# # Получить список всех subjects
# curl -X GET http://localhost:8081/subjects
# nshokurov@MB-YLV2KQ4C ~
# % curl -X GET http://localhost:8081/subjects                                                    [2025-09-03 15:56:37]
# ["persons-topic-value","your_topic-value"]%    

# # Получить все версии схемы для subject
# curl -X GET http://localhost:8081/subjects/vertica_datacontract/versions

# # Получить конкретную версию схемы
# curl -X GET http://localhost:8081/subjects/vertica_datacontract/versions/1
# echo $(curl -X GET http://localhost:8081/subjects/vertica_datacontract/versions/1) 

# http://localhost:8081/subjects/your_topic-value/versions/1

# # Проверить совместимость новой схемы
# curl -X POST -H "Content-Type: application/vnd.schemaregistry.v1+json" \
#   --data '{
#     "schema": "{\"type\":\"record\",\"name\":\"User\",\"fields\":[{\"name\":\"id\",\"type\":\"int\"},{\"name\":\"name\",\"type\":\"string\"},{\"name\":\"email\",\"type\":\"string\"},{\"name\":\"age\",\"type\":[\"null\",\"int\"],\"default\":null}]}"
#   }' \
#   http://localhost:8081/compatibility/subjects/users-value/versions/latest



import importlib
import inspect
import logging
import os
# from pathlib import Path
import pathlib
import shutil
import subprocess
import sys
import tempfile
import types
# import vertica_datacontract_pb2__created
import zipfile
import io

from datacontract.data_contract import DataContract

import click
import requests
from confluent_kafka import Consumer, Producer
from confluent_kafka.schema_registry import SchemaRegistryClient
from confluent_kafka.schema_registry.protobuf import ProtobufSerializer
from confluent_kafka.serialization import MessageField, SerializationContext
from google.protobuf.message import Message
from grpc_tools import protoc
# from person_pb2 import Person
from requests.auth import HTTPBasicAuth

from src.settings import (BOOTSTRAP_PORT, BOOTSTRAP_SERVER, KAFKA_TOPIC,
                          SCHEMA_REGISTRY_HOST, SCHEMA_REGISTRY_PORT)

log = logging.getLogger(name="").getChild(suffix=__name__)


def send_to_kafka(
    proto_message,
    bootstrap_servers: str = f"{BOOTSTRAP_SERVER}:{BOOTSTRAP_PORT}",
    topic: str = KAFKA_TOPIC,
    key=None,
):

    protobuf_serializer: ProtobufSerializer = ProtobufSerializer(
        msg_type=proto_message.__class__,
        schema_registry_client=SchemaRegistryClient(
            conf={"url": f"{SCHEMA_REGISTRY_HOST}:{SCHEMA_REGISTRY_PORT}"}
        ),
    )

    producer: Producer = Producer(
        {
            "bootstrap.servers": bootstrap_servers,
            "client.id": "python-producer",
        }
    )

    value = protobuf_serializer(
        message=proto_message,
        ctx=SerializationContext(topic=topic, field=MessageField.VALUE),
    )

    producer.produce(
        topic=topic,
        # key=key,
        value=value,
    )
    producer.flush()


SEND_TO_KAFKA_TXT: str = """
def send_to_kafka(
    proto_message,
    bootstrap_servers: str,
    topic: str,
    key=None,
):

    protobuf_serializer: ProtobufSerializer = ProtobufSerializer(
        msg_type=proto_message.__class__,
        schema_registry_client=SchemaRegistryClient(
            conf={"url": f"{SCHEMA_REGISTRY_HOST}:{SCHEMA_REGISTRY_PORT}"}
        ),
    )

    producer: Producer = Producer(
        {
            "bootstrap.servers": bootstrap_servers,
            "client.id": "python-producer",
        }
    )

    value = protobuf_serializer(
        message=proto_message,
        ctx=SerializationContext(topic=topic, field=MessageField.VALUE),
    )

    producer.produce(
        topic=topic,
        # key=key,
        value=value,
    )
    producer.flush()

"""


class ModuleBuilder:

    def __init__(self): ...


    def create_wheel(
        self,
        proto_file_name: str = "vertica_datacontract_pb2",
        module_folder: str = "vertica_datacontract",
        version: str = "0.1.8",
    ):

        # VERSION: str = "0.1.8"

        # MODULE_NAME: str = "vertica-datacontract-tool"
        # MODULE_NAME: str = "vertica_datacontract_tool"

        # у нас должен быть файл proto.py:
        # PROTO_FILE_NAME: str = "vertica_datacontract_pb2"

        # MODULE_FOLDER: str = "vertica_datacontract"

        # pyproject_toml_content: str = """
        # [build-system]
        # requires = ["setuptools", "wheel"]
        # build-backend = "setuptools.build_meta"
        # """

        setup_file_content: str = f"""
from setuptools import setup, find_packages

setup(
    name="{module_folder}",
    version="{version}",# my_package.__version__,
    # author=my_package.__author__,
    packages=find_packages(),
    install_requires=[
        "protobuf>=3.20.0",
        "build>=1.3.0",
        "confluent-kafka[all]>=2.11.1",
        "pip>=25.2",
        ],

)
"""

        with tempfile.TemporaryDirectory() as tmpdirname:
            setup_file: str = os.path.join(tmpdirname, "setup.py")

            with open(setup_file, "w") as f:
                f.write(setup_file_content)

            module_folder_path: str = os.path.join(tmpdirname, module_folder)
            os.makedirs(module_folder_path)  # , exist_ok=True)

            init_file: str = os.path.join(module_folder_path, "__init__.py")

            with open(f"{proto_file_name}.py", "r") as proto1:
                with open(init_file, "w") as f:
                    f.write(proto1.read())
                    f.write("\n\n")
                    f.write(SEND_TO_KAFKA_TXT)

            with open(init_file, "r") as f:
                print("its __init__.py")
                print(f.read())
            with open(setup_file, "r") as f:
                print("its setup.py")
                print(f.read())

            subprocess.run(
                args=["python", "setup.py", "bdist_wheel"],
                check=True,
                cwd=tmpdirname,  # указать working directory
            )

            dist_folder: str = os.path.join(tmpdirname, "dist")
            wheels: list[str] = [
                os.path.join(dist_folder, filepath)
                for filepath in os.listdir(dist_folder)
            ]
            print({"wheels": wheels})

            for wheel in wheels:
                subprocess.run(
                    args=["python", "-m", "pip", "install", wheel],
                    check=True,
                    cwd=tmpdirname,  # указать working directory
                )

            real_dir = os.getcwd()
            for wheel in wheels:
                print(f"try copy2 {wheel}")
                shutil.copy2(src=wheel, dst=real_dir)

    def publish_package(
        self,
        file_path: str = "",
        nexus_url: str = "",
        nexus_repo_path: str = "",
        username: str = "",
        password: str = "",
    ):
        with open(file=file_path, mode="rb") as file:

            response: requests.Response = requests.put(
                url=f"{nexus_url}/repository/{nexus_repo_path}/{os.path.basename(p=file_path)}",
                auth=HTTPBasicAuth(username=username, password=password),
                data=file,
                headers={"Content-Type": "application/json"},
                timeout=20,
            )
            response.raise_for_status()
            click.echo(message=f"The file has been successfully uploaded to Nexus: {response.url}")

    def validate_custom(self, filename: str):

        data_contract: DataContract = DataContract(
            data_contract_file=f"{filename}.yaml"
        )
        data_contract.lint()

        click.echo(message=f" hello is custom validate {filename}.yaml")

    def create_yaml_from_sql(self, filename: str = "test"):
        # TODO: может можно как-то питоновским кодом сделать, без subprocess

        create_yaml: str = (
            f"""uv run datacontract import --format sql --source {filename}.sql --output {filename}.yaml"""
        )
        click.echo(message=create_yaml)

        result: subprocess.CompletedProcess = subprocess.run(
            args=create_yaml,
            shell=True,
            executable="/bin/bash",  # или '/bin/zsh'
            capture_output=True,
            text=True,
            check=True,
        )
        click.echo(message=result)

    def create_proto_from_yaml(self, filename: str = "vertica_datacontract"):
        """нужен файлик your-datacontract.yaml"""

        data_contract: DataContract = DataContract(
            data_contract_file=f"{filename}.yaml"
        )
        data_contract.lint()

        file_to_create: str = f"{filename}.proto"

        with open(file=file_to_create, mode="wb") as f:
            f.write(data_contract.export(export_format="protobuf")["protobuf"].encode())
        
        click.echo(message=f"created file: {file_to_create}")


    def generate_python_code_from_proto(self, filename: str):
        # TODO: мне кажется здесь надо принимать не название .proto файла, а сам файл в качестве аргумента

        protoc.main(["protoc", "--python_out=.", f"{filename}.proto"])
        click.echo(message=f"created file {filename}_pb2.py")



import json
import logging
import re
import subprocess
import sys
from pathlib import Path
from datacontract.data_contract import DataContract

import click
import requests

from src.settings import SCHEMA_REGISTRY_HOST, SCHEMA_REGISTRY_PORT

# from src import commands


log = logging.getLogger(name="").getChild(suffix=__name__)


class ContractValidator:
    def __init__(self): ...


    def validate_custom(self, filename: str):
        

        data_contract: DataContract = DataContract(
            data_contract_file=f"{filename}.yaml"
        )
        data_contract.lint()

        click.echo(message=f" hello is custom validate {filename}")


        
    def validate_schema_registry(
        self,
        subject_name: str,
        version: str,
        filename: str,
    ):

        url: str = (
            f"{SCHEMA_REGISTRY_HOST}:{SCHEMA_REGISTRY_PORT}/compatibility/subjects/{subject_name}/versions/{version}"
        )

        # proto_file_name: str = "person.proto"
        data_contract: DataContract = DataContract(
            data_contract_file=f"{filename}.yaml"
        )
        data_contract.lint()
        exported: dict = data_contract.export(export_format="protobuf")
        print(exported["protobuf"])
        # # Сохранение результата в файл
        # with open(file=f"{filename}.proto", mode="wb") as f:
        #     f.write(exported["protobuf"].encode())


        schema_content = exported["protobuf"]
        # with open(file=f"{proto_file}.proto", mode="r") as f:
        #     schema_content: str = f.read()
        # print({"schema_content": schema_content})

        url: str = (
            f"{SCHEMA_REGISTRY_HOST}:{SCHEMA_REGISTRY_PORT}/compatibility/subjects/{subject_name}/versions/{version}"
        )
        click.echo(message={"url": url})
        response: requests.Response = requests.post(
            url=url,
            headers={"Content-Type": "application/vnd.schemaregistry.v1+json"},
            data=json.dumps(obj={"schema": schema_content, "schemaType": "PROTOBUF"}),
            timeout=20,
        )
        click.echo(message=response.text)



import logging
import sys
from pathlib import Path

import click

from src.helpers import ModuleBuilder
from src.settings import (BOOTSTRAP_PORT, BOOTSTRAP_SERVER, KAFKA_TOPIC,
                          SCHEMA_REGISTRY_HOST, SCHEMA_REGISTRY_PORT)


# Добавляем текущую директорию в путь импорта
current_dir = Path(__file__).parent.absolute()
sys.path.insert(0, str(object=current_dir))


log: logging.Logger = logging.getLogger(name=__name__)
log.setLevel(level=logging.DEBUG)


# # Получить список всех subjects
# curl -X GET http://localhost:8081/subjects
# nshokurov@MB-YLV2KQ4C ~
# % curl -X GET http://localhost:8081/subjects                                                    [2025-09-03 15:56:37]
# ["persons-topic-value","your_topic-value"]%

# # Получить все версии схемы для subject
# curl -X GET http://localhost:8081/subjects/vertica_datacontract/versions

# # Получить конкретную версию схемы
# curl -X GET http://localhost:8081/subjects/vertica_datacontract/versions/1
# echo $(curl -X GET http://localhost:8081/subjects/vertica_datacontract/versions/1)


@click.group()
@click.pass_context
def cli(
    ctx,
):
    ctx.ensure_object(dict)
    print("is cli")


@cli.command()
@click.option("--filename")
@click.option("--subject-name")
def publish_schema_registry(
    filename: str,
    subject_name: str,
):
    """
    uv run --env-file .env python -m src publish-schema-registry --filename "vertica_datacontract" --subject-name vertica_datacontract

    """
    ModuleBuilder().publish_schema_registry(
        filename=filename, subject_name=subject_name
    )


@cli.command()
@click.option("--filename", default="vertica_datacontract")
@click.option(
    "--subject-name",
    default="vertica_datacontract",
)
@click.option("--version", default="latest")
def validate_schema_registry(
    filename: str = "vertica_datacontract",
    subject_name: str = "vertica_datacontract",
    version: str = "latest",
):
    """
    vertica_datacontract.proto

    uv run --env-file .env python -m src validate-schema-registry --filename "vertica_datacontract" --subject-name vertica_datacontract
    """
    ModuleBuilder().validate_custom(filename=filename)

    ModuleBuilder().validate_schema_registry(
        subject_name=subject_name, version=version, filename=filename
    )


@cli.command()
@click.option("--filename", required=False, type=str, help="Название файла")
def validate_custom(filename: str):
    """
    uv run --env-file .env python -m src validate-custom --filename vertica_datacontract

    """
    ModuleBuilder().validate_custom(filename=filename)


@cli.command()
@click.option("--filename", required=True, type=str, help="Название файла")
def create_yaml_from_sql(filename: str):
    """
    нужен ddl.sql

    uv run --env-file .env python -m src create-yaml-from-sql --filename vertica_datacontract
    """
    ModuleBuilder().create_yaml_from_sql(filename=filename)


# не уверен, что эта команда нужна
@cli.command()
@click.option("--filename", required=True, type=str, help="Название файла")
def create_proto_from_yaml(filename: str):
    """
    нужен your-datacontract.yaml

    uv run --env-file .env python -m src create-proto-from-yaml --filename vertica_datacontract

    """
    ModuleBuilder().create_proto_from_yaml(filename=filename)


@cli.command()
@click.option("--filename", required=True, type=str, help="Название файла")
def generate_python_code_from_proto(filename: str):
    """
    uv run --env-file .env python -m src generate-python-code-from-proto --filename vertica_datacontract
    """
    ModuleBuilder().generate_python_code_from_proto(filename=filename)


@cli.command()
@click.option("--wheel-version", required=True)
@click.option("--proto-file-name", default="vertica_datacontract_pb2")
@click.option("--module-folder", default="vertica_datacontract")
def create_wheel(
    wheel_version: str,
    proto_file_name: str = "vertica_datacontract_pb2",
    module_folder: str = "vertica_datacontract",
):
    """
    uv run --env-file .env python -m src create-wheel --proto-file-name vertica_datacontract_pb2 --wheel-version 0.1.9 --module-folder vertica_datacontract
    """
    ModuleBuilder().create_wheel(
        proto_file_name=proto_file_name,
        module_folder=module_folder,
        version=wheel_version,
    )


@cli.command()
@click.option("--module-name", required=False, type=str)
@click.option("--filepath", required=False, type=str)
@click.option("--nexusurl", required=False, type=str)
@click.option("--nexusrepopath", required=False, type=str)
@click.option("--username", required=False, type=str)
@click.option("--password", required=False, type=str)
def publish_package(
    filepath: str, nexusurl: str, nexusrepopath: str, username: str, password: str
):
    """
    не работает, вручную в нексус залил
    https://nexus.k8s-analytics.ostrovok.in/#browse/browse:datacontract_pypi:datacontract-helper%2F0.1.32%2Fdatacontract_helper-0.1.32-py3-none-any.whl
    но как в коде залить не понимаю

    """
    ModuleBuilder().publish_package(
        file_path=filepath,
        nexus_url=nexusurl,
        nexus_repo_path=nexusrepopath,
        username=username,
        password=password,
    )



import io
import os
import subprocess
import sys

import requests
from requests.auth import HTTPBasicAuth
from twine.commands.upload import upload

NEXUS_URL: str = os.environ["NEXUS_URL"]
NEXUS_USERNAME: str = os.environ["NEXUS_USERNAME"]
NEXUS_PASSWORD: str = os.environ["NEXUS_PASSWORD"]


class NexusPublisher:
    def __init__(self, package_path: str):
        result: subprocess.CompletedProcess = subprocess.run(
            args=[
                "uv",
                "run",
                "twine",
                "upload",
                "--repository-url",
                os.environ["NEXUS_URL"],
                "--username",
                os.environ["NEXUS_USERNAME"],
                "--password",
                os.environ["NEXUS_PASSWORD"],
                "dist/*",
            ],
            capture_output=True,
            text=True,
        )
        print(result.stdout)
        # uv run python -m twine upload dist/*
        # uv run twine upload --repository-url repository_url --username username --password password  dist/*
